#### Application and library build and deployment

Since the initial ProGuard support was added to Xamarin.Android, an
undocumented `$(ProguardConfigFiles)` MSBuild property could be used
as an alternative to using the `@(ProguardConfiguration)` item group:

```xml
<PropertyGroup>
  <ProguardConfigFiles>
    {sdk.dir}tools\proguard\proguard-android.txt;
    {intermediate.common.xamarin};
    {intermediate.references};
    {intermediate.application};
    @(ProguardConfiguration);
  </ProguardConfigFiles>
</PropertyGroup>
```

Xamarin.Android then used string replacements such as `{sdk.dir}`
instead of standard MSBuild. The string replacement behavior has been
removed from Xamarin.Android.

If your project is currently using `$(ProguardConfigFiles)` such as:

```xml
<PropertyGroup>
  <ProguardConfigFiles>
    proguard1.cfg;
    proguard2.cfg;
  </ProguardConfigFiles>
</PropertyGroup>
```

It is recommended to use the `@(ProguardConfiguration)` item group
instead:

```xml
<ItemGroup>
  <ProguardConfiguration Include="proguard1.cfg" />
  <ProguardConfiguration Include="proguard2.cfg" />
</ItemGroup>
```

In Visual Studio, you can add text files with a `Build Action` of
`ProguardConfiguration` as [documented][0].

[0]: https://docs.microsoft.com/xamarin/android/deploy-test/release-prep/proguard#customizing-proguard
